/********************************************************
 *  Copyright (C) 2003 SOFTWARE AG, All Rights reserved
 ********************************************************
 * - list all collections in a Tamino database
 ********************************************************/

#include <stdio.h>
#include <string.h>
#include "expat.h" /* use expat XML parser for parsing Tamino responses */
#include "tac.h"

#define checkERROR(a)	if(a != TAC_SUCCESS) goto error

#ifdef _WIN32
#	define strcasecmp(a, b) _stricmp(a, b)  /* non-case sensitive comparison */
#endif


static void CollectionElement(void *userData, const char *name, const char **atts)
{
  int i = 0;
  int *number = userData;

  if (strcasecmp(name, "tsd:collection") == 0) {
	 *number += 1;
	 printf("- info: Collection #%d:", *number);
	 while (atts[i] != NULL) {
		if (strcasecmp(atts[i], "name") == 0) {
	printf("\t%s\n", atts[i+1]);
		}
		i+=2;
	 }
  }
}

static int find_collections(TAC_HANDLE handle)
{
  XML_Parser parser;
  int ret_code1, ret_code2, number = 0;
  const char *response;

  /* query the db */
  ret_code1 = tac_xql(handle, "ino:collection", "tsd:collection");
  if (ret_code1 == TAC_SUCCESS) {

	 /* process the response */
	 parser = XML_ParserCreate(NULL);
	 XML_SetUserData(parser, &number);
	 XML_SetElementHandler(parser, CollectionElement, NULL);
	 (void) tac_last_xml_response(handle, &response);
	 if ((ret_code2 = XML_Parse(parser, response, strlen(response), 1)) == 0) {
		fprintf(stderr, "- error: while parsing Tamino response (%d)\n", ret_code2);
	 }
	 XML_ParserFree(parser);
  }
  return ret_code1;
}


int main()
{
	TAC_HANDLE handle;
	const char *response;

	/* initialize the API */
	if ((handle = tac_init("mydb")) < 0) goto error;

	/* ask for all collections in the Tamino database */
	checkERROR(find_collections(handle));

	/* finalize the API */
	checkERROR(tac_end(handle));

	return 0;

/* error handling */
error:
	(void) tac_get_messagetext(handle, &response);
	printf("%s\n", response);
	tac_end(handle);

	return 1;
}
