/********************************************************
 *  Copyright (C) 2003 SOFTWARE AG, All Rights reserved
 ********************************************************
 *  - load and retrieve data into/from Tamino database
 ********************************************************/

#include <stdio.h>
#include "tac.h"

#define checkERROR(a)	if(a != TAC_SUCCESS) goto error

int main()
{
	TAC_HANDLE handle;
	const char *response;

	/* initialize the API */
	if ((handle = tac_init("mydb")) < 0) goto error;

	/* load Tamino schema definition from file */
	checkERROR(tac_define_from_file(handle, "TAC_samples.tsd"));

	/* load XML entries from file into Tamino collection */
	checkERROR(tac_process_from_file(handle, "RealEstate", "TAC_samples.xml"));

	/* load non-XML data from file into Tamino collection */
	checkERROR(tac_load_from_file(handle, "RealEstate", "images/sample1.jpg", "TAC_sample1.jpg", "image/jpeg"));
		checkERROR(tac_load_from_file(handle, "RealEstate", "images/sample2.jpg", "TAC_sample2.jpg", "image/jpeg"));

	/* check for entries in the Tamino collection */
	checkERROR(tac_xql(handle, "RealEstate", "/images"));

	/* display the Tamino response */
	checkERROR(tac_last_xml_response(handle, &response));
	printf("%s\n", response);

	/* undefine the above defined schema (clean-up) */
	checkERROR(tac_undefine(handle, "RealEstate"));

	/* finalize the API */
	checkERROR(tac_end(handle));

	return 0;

/* error handling */
error:
	(void) tac_get_messagetext(handle, &response);
	printf("%s\n", response);
	tac_end(handle);

	return 1;
}
